import requests


class Client:
    def __init__(self, token) -> None:
        self.token = token

    def request(self, method: str, route: str, **kwargs) -> requests.Response:
        if "headers" not in kwargs:
            kwargs["headers"] = {}
        kwargs["headers"]["Authorization"] = "Bearer " + self.token
        response = getattr(requests, method)(route, **kwargs)
        if response.status_code == 401:
            raise Exception("Not authorized! Please verify if your token is correct and if you have access to this service.")
        if not response:
            raise Exception(
                f"[Error] Cannot query server for route {route}. {response.content}")
        return response



from .config import api_urls
import requests


def generate_token(username="", password="", api_url=api_urls["accounts"]) -> str:
    from getpass import getpass
    data = {
        "username": username or input("Username: "),
        "password": password or getpass("Password: "),
    }
    response = requests.post(f"{api_url}/login", json=data)
    if response and "access_token" in response.json():
        return response.json()["access_token"]
    raise Exception("[Error] Authentication failed.")



from __future__ import annotations
from multiprocessing import Pool
from .client import Client
from .config import api_urls
from typing import Union


class Biofeature:
    def __init__(self, client: Client) -> None:
        self.client = client

    def call(self, img: str, cpf: Union[None, str] = None) -> dict:
        response = self.client.request(
            "post", f"{api_urls['biofeatureai']}/predict", files={"file": open(img, 'rb')}, json={"cpf": cpf})
        if not response.status_code == 201:
            raise Exception(f"[Error] Prediction failed. {response.content}")
        return response.json()

    def multiple_call(self, items: list[dict[str, str]]) -> list[dict]:
        for item in items:
            if not "img" in item:
                raise Exception(f"[Error] Missing img key on dict {item}.")
        with Pool() as pool:
            response = pool.starmap(self.call, [(
                item["img"], item["cpf"] if "cpf" in item else None) for item in items])
        return response
